unit CustomForm;

interface

uses
  Windows,WinProcs, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, Menus,
  registry, ExtCtrls, Typinfo;

type
  TObjectEvents = class
  public
     FObject : TObject;
     FMouseDownEvent : TMouseEvent;
  end;

  TfmCustomForm = class(TForm)
    pupOpcoes: TPopupMenu;
    miPropriedades: TMenuItem;
    miFinalizar: TMenuItem;
    N1: TMenuItem;
    miReset: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure miFinalizarClick(Sender: TObject);
    procedure miResetClick(Sender: TObject);
    procedure miPropriedadesClick(Sender: TObject);
  private
    FReset         : Boolean;   // Indica se foi ressetado as posies dos objetos
    FMovendo       : Boolean;   // Inidica se um objeto esta sendo movido
    FObjectsEvents : TList;     // Lista com os objetos e ponteiros de eventos
    FObjCustom     : TControl;  // Ponteiro para o objeto que esta sendo movido  
    FObjHandle     : HWND;      // Ponteiro para o manipulador de janela
    FPoint         : TPoint;    // Armazena o ponto em que o mouse foi clicado
    FObjSelecionado: TComponent;// Ponteiro para o objeto selecionado para alterao das propriedades

    procedure AlteraCursor(AContainer : TWinControl; ACursor : TCursor);
    procedure AlteraEventos(AComponent : TComponent);


  protected
    FCustomizando : Boolean; // Indica se esta sendo realizado a customizao da tela

    procedure InitCustom; virtual;
    procedure FinCustom; virtual;
    procedure ResetPositions; virtual;
    procedure SelecionaObjeto(AHandle : HWND; AObject : TComponent);

    procedure AppMessage(var Msg: TMsg; var Handled: Boolean);
  public
    { Public declarations }
    procedure MouseDownCustom(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  end;

const
  SC_MOVE = $F012;

var
  fmCustomForm: TfmCustomForm;

implementation

uses PropriedadesForm;

{$R *.DFM}

// Mtodo que finaliza a customizao do formulrio
procedure TfmCustomForm.FinCustom;
begin
   // marca que no esta customizando
   FCustomizando := False;
   // retorna o cursor ao ponteiro padro
   AlteraCursor(Self,crDefault);
   // remove o evento de tratamento das mensagens
   Application.OnMessage := Nil;
end;

// Mtodo construtor do formulrio
procedure TfmCustomForm.FormCreate(Sender: TObject);
var
  i: Integer;
  RegIni : TRegIniFile;
  PropValue : Variant;
  Component : TComponent;
begin
   // Inicializa as variveis
   FCustomizando  := False;
   FReset         := False;
   FObjectsEvents := TList.Create;

   // Abre o registro para ler as configuraes
   RegIni := TRegIniFile.Create(Application.Title);

   // Percorre todos os componentes do formulrio
   for i := 0 to ComponentCount -1 do
   begin
      Component := Components[i];

      // Se for um controle
      if Components[i] is TControl then
      begin
         with Components[i] as TControl Do
         begin
            Left := RegIni.ReadInteger(Self.ClassName,Name + '.' + 'Left',Left);
            Top  := RegIni.ReadInteger(Self.ClassName,Name + '.' + 'Top' ,Top);

            // Obtem o valor da propriedade Align
            try
               PropValue := RegIni.ReadInteger(Self.ClassName,Name + '.' + 'Align',-1);

               if PropValue > -1 then
                  SetPropValue(Component,'Align',TAlign(PropValue));
            except
            end;

            // Obtem o valor da propriedade Height
            try
               PropValue := RegIni.ReadInteger(Self.ClassName,Name + '.' + 'Height',Height);

               SetPropValue(Component,'Height',PropValue);
            except
            end;

            // Obtem o valor da propriedade Width
            try
               PropValue := RegIni.ReadInteger(Self.ClassName,Name + '.' + 'Width',Width);

               SetPropValue(Component,'Width',PropValue);
            except
            end;

            // Obtem o valor da propriedade TabOrder
            try
               PropValue := RegIni.ReadInteger(Self.ClassName,Name + '.' + 'TabOrder',-1);

               if PropValue > -1 then
                  SetPropValue(Component,'TabOrder',PropValue);
            except
            end;
         end;
      end;
   end;

   // Fecha o registro
   RegIni.free;
end;

// Mtodo responsvel por alterar o cursor nos controles
procedure TfmCustomForm.AlteraCursor(AContainer: TWinControl;  ACursor : TCursor);
var
   i  : Integer;
   Component : TComponent;
begin
   // Percorre todos os componentes do Container
   for i := 0 to AContainer.ComponentCount - 1 do
   begin
      Component := AContainer.Components[i];

      // Se o componente for um container
      if Component is TWinControl then
      begin
         // Altera o cursor dos componentes do container
         AlteraCursor(TWinControl(Component),ACursor);
         // altera o cursor do controle
         with Component as TWinControl do
            Cursor := ACursor;
      end;

      // Se o componente for um controle
      if Component is TControl then
      begin
         with Component as TControl do
         begin
            // altera o cursor do controle
            Cursor := ACursor;
            // Altera os eventos
            AlteraEventos(Component);
         end;
      end;
   end;
end;

procedure TfmCustomForm.AlteraEventos(AComponent: TComponent);
var
   j : Integer;
   ObjEvents : TObjectEvents;
begin
   if (not (AComponent is TWinControl)) then
   begin
      try
         // Se estiver iniciando a customizao
         if AComponent.MethodName(GetMethodProp(AComponent,'OnMouseDown').Code) <> 'MouseDownCustom' then
         begin
            // Cria o Objeto
            ObjEvents := TObjectEvents.Create;
            // Atribui os ponteiros para o objeto e eventos
            ObjEvents.FObject := AComponent;
            ObjEvents.FMouseDownEvent := TForm(AComponent).OnMouseDown;
            // Guarda na lista de eventos dos objetos
            FObjectsEvents.Add(ObjEvents);

            // liga o evento MouseDown
            TForm(AComponent).OnMouseDown := MouseDownCustom;
         end
         else
         begin
            // Percorre os objetos de eventos
            for j := 0 to FObjectsEvents.Count - 1 do
            begin
               ObjEvents := TObjectEvents(FObjectsEvents.Items[j]);

               if ObjEvents.FObject = AComponent then
               begin
                  // Retorna os eventos
                  TForm(AComponent).OnMouseDown := ObjEvents.FMouseDownEvent;
                  // Deleta o objeto da lista
                  FObjectsEvents.Delete(j);
                  // Destroi o objeto
                  ObjEvents.Free;
                  // Para o loop
                  Break;
               end;
            end;
         end;
      except
         TControl(AComponent).Cursor := crDefault;
      end;
   end;
end;

// Mtodo responsvel por processar as mensagens
procedure TfmCustomForm.AppMessage(var Msg: TMsg; var Handled: Boolean);
var
   Point : TPoint;
begin
   // Se esta customizando o formulrio
   if FCustomizando then
   begin
      // Se esta movendo o mouse
      if  (Msg.message = WM_MOUSEMOVE) then
      begin
         // Se esta movendo um objeto
         if FMovendo then
         begin
            if (FObjCustom <> Nil) then
            begin
               GetCursorPos(Point);

               if FObjCustom.Parent = Self then
                  Point := ScreenToClient(Point)
               else
                  Point := FObjCustom.Parent.ScreenToClient(Point);

               SetPropValue(FObjCustom,'Left',Point.X - FPoint.X);
               SetPropValue(FObjCustom,'Top' ,Point.Y - FPoint.Y);
            end
            else
            begin
               // Captura o objeto
               WinProcs.ReleaseCapture;
               // Envia a mensagem para movimentar o controle
               PostMessage(FObjHandle,WM_SysCommand, SC_MOVE, 0 );
            end;

            // Informa que a mensagem foi processada
            Handled := True;
         end;
      end;

      if  (Msg.message = WM_LBUTTONDOWN) then
      begin
         FObjHandle := Msg.hwnd;
         // Captura o objeto
         WinProcs.ReleaseCapture;
         // Guarda que iniciou o movimento
         FMovendo := True;
      end;

      if  (Msg.message = WM_LBUTTONUP) then
      begin
         FObjHandle := 0;
         FObjCustom := Nil;
         FMovendo   := False;

         // Informa que a mensagem foi processada
         Handled := True;
      end;

      // Se foi pressionado o boto direito do mouse
      if  (Msg.message = WM_RBUTTONDOWN) then
      begin
         FObjHandle := Msg.hwnd;
      end;

      // Se foi solto o boto direito do mouse
      if Msg.message = WM_RBUTTONUP then
      begin
         if WinProcs.ReleaseCapture then
         begin
            miPropriedades.Enabled := (FObjHandle <> Self.Handle) or (FObjCustom <> Nil);

            GetCursorPos(Point);

            SelecionaObjeto(FObjHandle,FObjCustom);

            pupOpcoes.Popup(Point.X,Point.Y);

            FObjCustom := Nil;
            FObjHandle := 0;
         end;
      end;
   end;
end;

// Mtodo de destruio do form
procedure TfmCustomForm.FormDestroy(Sender: TObject);
var
  i         : Integer;
  RegIni    : TRegIniFile;
  PropValue : Variant;
  Component : TComponent;
begin
   // Se no foi Resetado as posies
   if not FReset then
   begin
      // Abre o registro
      RegIni := TRegIniFile.Create(Application.Title);
      // Percorre todos os componentes do formulrio
      for i := 0 to ComponentCount -1 do
      begin
         Component := Components[i];

         // Se for um controle
         if Components[i] is TControl then
         begin
            with Components[i] as TControl Do
            begin
               // Salva as propriedades do controle
               RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Left',Left);
               RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Top' ,Top);

               // Obtem o valor da propriedade Align
               try
                  PropValue := GetPropValue(Component,'Align');

                  if PropValue = 'alNone' then
                     RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Align',0);
                  if PropValue = 'alTop' then
                     RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Align',1);
                  if PropValue = 'alBottom' then
                     RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Align',2);
                  if PropValue = 'alLeft' then
                     RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Align',3);
                  if PropValue = 'alRight' then
                     RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Align',4);
                  if PropValue = 'alClient' then
                     RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Align',5);
               except
                  RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Align',-1);
               end;

               // Obtem o valor da propriedade Height
               try
                  PropValue := GetPropValue(Component,'Height');

                  RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Height',PropValue);
               except
                  RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Height',0);
               end;

               // Obtem o valor da propriedade Width
               try
                  PropValue := GetPropValue(Component,'Width');

                  RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Width',PropValue);
               except
                  RegIni.WriteInteger(Self.ClassName,Name + '.' + 'Width',0);
               end;

               // Obtem o valor da propriedade TabOrder
               try
                  PropValue := GetPropValue(Component,'TabOrder');

                  RegIni.WriteInteger(Self.ClassName,Name + '.' + 'TabOrder',PropValue);
               except
                  RegIni.WriteInteger(Self.ClassName,Name + '.' + 'TabOrder',-1);
               end;
            end;
         end;
      end;

      // Fecha o registro
      RegIni.Free;
   end;

   // Destroi o objeto
   FObjectsEvents.Free;
end;

// Mtodo que inicia a customizao do formulrio
procedure TfmCustomForm.InitCustom;
begin
   // Marca que esta customizando
   FCustomizando := True;
   // Altera o cursor dos componentes
   AlteraCursor(Self, crSizeAll);
   // Liga o evento de processamento das mensagens
   Application.OnMessage := AppMessage;
end;

// Evento disparado quando o boto do mouse  pressionado
procedure TfmCustomForm.MouseDownCustom(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
   // Guarda o objeto que esta sendo customizado
   FObjCustom := TControl(Sender);
   FPoint.X := X;
   FPoint.Y := Y;
end;

// Reseta as posies dos controles do formulrio
procedure TfmCustomForm.ResetPositions;
var
   RegIni : TRegIniFile;
begin
   // Marca que esta resetando
   FReset := True;
   // Abre o registro
   RegIni := TRegIniFile.Create(Application.Title);
   // Apaga a seo deste formulrio
   RegIni.EraseSection(ClassName);
   // Fecha o registro
   RegIni.Free;
end;

// Seleciona o objeto que ser alterado as propriedades
procedure TfmCustomForm.SelecionaObjeto(AHandle: HWND; AObject: TComponent);
var
   i : Integer;
begin
   if AObject <> Nil then
      FObjSelecionado := AObject
   else
   begin
      for i := 0 to ComponentCount - 1 do
      begin
         if Components[i] is TWinControl then
         begin
            if TWinControl(Components[i]).Handle = AHandle then
            begin
               FObjSelecionado := Components[i];
               //
               Break;
            end;
         end;
      end;
   end;
end;

// Evento do popup menu
procedure TfmCustomForm.miFinalizarClick(Sender: TObject);
begin
   // Finaliza as alteraes
   FinCustom;
end;

// Evento do popup menu
procedure TfmCustomForm.miPropriedadesClick(Sender: TObject);
var
   PropValue : Variant;
begin
   // Desliga o controle de mensagens
   Application.OnMessage := nil;
   // Cria o formulrio de propriedades
   fmPropriedades := TfmPropriedades.Create(Self);

   // Obtem o valor da propriedade Align
   try
      PropValue := GetPropValue(FObjSelecionado,'Align');

      if PropValue = 'alNone' then
         fmPropriedades.cmbAlinhamento.ItemIndex := 0;
      if PropValue = 'alTop' then
         fmPropriedades.cmbAlinhamento.ItemIndex := 1;
      if PropValue = 'alBottom' then
         fmPropriedades.cmbAlinhamento.ItemIndex := 2;
      if PropValue = 'alLeft' then
         fmPropriedades.cmbAlinhamento.ItemIndex := 3;
      if PropValue = 'alRight' then
         fmPropriedades.cmbAlinhamento.ItemIndex := 4;
      if PropValue = 'alClient' then
         fmPropriedades.cmbAlinhamento.ItemIndex := 5;
   except
      fmPropriedades.cmbAlinhamento.ItemIndex := -1;
      fmPropriedades.cmbAlinhamento.Enabled   := False;
   end;

   // Obtem o valor da propriedade Height
   try
      PropValue := GetPropValue(FObjSelecionado,'Height');

      fmPropriedades.udAltura.Position := PropValue;
   except
      fmPropriedades.edtAltura.Enabled := False;
      fmPropriedades.udAltura.Enabled  := False;
   end;

   // Obtem o valor da propriedade Width
   try
      PropValue := GetPropValue(FObjSelecionado,'Width');

      fmPropriedades.udLargura.Position := PropValue;
   except
      fmPropriedades.edtLargura.Enabled := False;
      fmPropriedades.udLargura.Enabled  := False;
   end;

   // Obtem o valor da propriedade TabOrder
   try
      PropValue := GetPropValue(FObjSelecionado,'TabOrder');

      fmPropriedades.edtTabOrder.Text := PropValue;
   except
      fmPropriedades.edtTabOrder.Enabled := False;
   end;

   // Mostra a tela e espera pressionar o boto OK
   if fmPropriedades.ShowModal = mrOk then
   begin
      with fmPropriedades do
      begin
         // Altera a propriedade Align
         if cmbAlinhamento.Enabled then
         begin
            SetPropValue(FObjSelecionado,'Align',TAlign(cmbAlinhamento.ItemIndex));
         end;

         // Altera a propriedade Height
         if edtAltura.Enabled then
         begin
            SetPropValue(FObjSelecionado,'Height',edtAltura.Text);
         end;

         // Altera a propriedade Width
         if edtLargura.Enabled then
         begin
            SetPropValue(FObjSelecionado,'Width',edtLargura.Text);
         end;

         // Altera a propriedade TabOrder
         if edtTabOrder.Enabled then
         begin
            SetPropValue(FObjSelecionado,'TabOrder',edtTabOrder.Text);
         end;
      end;
   end;

   // Destroi a tela de propriedades
   fmPropriedades.Free;

   // Liga o controle de mensagens
   Application.OnMessage := AppMessage;
end;

// Evento do popup menu
procedure TfmCustomForm.miResetClick(Sender: TObject);
begin
   // Reseta as posies e finaliza as alteraes
   ResetPositions;
   FinCustom;

   Application.MessageBox('Os campos voltaro a posio inicial na prxima vez que essa tela for aberta.','Ateno',MB_OK + MB_ICONEXCLAMATION);
end;

end.

